import requests

class BaseAPI:
    def __init__(self, base_url, access_key, xml=False, format_csv=False):
        self.base_url = base_url
        self.access_key = access_key
        self.xml = xml
        self.format_csv = format_csv

    def _get(self, endpoint, params=None):
        url = f"{self.base_url}{endpoint}"
        if params is None:
            params = {}
        
        # Append mandatory parameters
        params['accessKey'] = self.access_key
        
        # Append conditional parameters
        if self.xml:
            params['xml'] = 'true'
        if self.format_csv:
            params['format'] = 'CSV'
        
        response = requests.get(url, params=params)
        response.raise_for_status()  # Raise HTTPError for bad responses (4xx, 5xx)
        
        # Determine the content type and handle the response accordingly
        content_type = response.headers.get('Content-Type', '')
        if 'application/json' in content_type:
           
            return response.json()
        elif 'application/xml' in content_type or 'text/xml' in content_type:
            
            return response.text
        elif 'text/csv' in content_type or 'application/csv' in content_type:
            
            return response.text
        else:
            return response.text


from .base_api import BaseAPI

class GetLastQuoteArray(BaseAPI):
    def GetLastQuoteArray(self, exchange, instrument_identifiers):
        endpoint = "GetLastQuoteArray/"
        params = {
            'exchange': exchange,
            'instrumentIdentifiers': instrument_identifiers
        }
        return self._get(endpoint, params)



from .base_api import BaseAPI

class GetInstrumentTypes(BaseAPI):
    def Get_InstrumentTypes(self,exchange):
        endpoint = "GetInstrumentTypes/"
        params = {
            'exchange': exchange
            
        }
        

        return self._get(endpoint,params)  



from .base_api import BaseAPI

class GetLastQuote(BaseAPI):
    def Get_LastQuote(self, exchange, instrument_identifier):
        endpoint = "GetLastQuote/"
        params = {
            'exchange': exchange,
            'instrumentIdentifier': instrument_identifier
        }
        return self._get(endpoint, params)


from .base_api import BaseAPI

class GetLastQuoteOptionGreeksChain(BaseAPI):
    
    def Get_LastQuoteOptionGreeksChain(self, exchange, product, expiry=None, option_type=None, strike_price=None):
        endpoint = "GetLastQuoteOptionGreeksChain/"
        params = {
            'exchange': exchange,
            'product': product
        }
        if expiry:
            params['expiry'] = expiry
        if option_type:
            params['optionType'] = option_type
        if strike_price:
            params['strikePrice'] = strike_price
        return self._get(endpoint, params) 



from .base_api import BaseAPI

class GetExchangeMessages(BaseAPI):
    def Get_Exchange_Messages(self,exchange):
        endpoint = "GetExchangeMessages/"
        params = {
            'exchange': exchange
        }
        

        return self._get(endpoint,params) 


from .base_api import BaseAPI

class GetExchangeSnapshot(BaseAPI):
    def Get_ExchangeSnapshot(self, exchange,  periodicity, period, instrumentType=None,nonTraded=None, from_timestamp=None, to_timestamp=None):
        endpoint = "GetExchangeSnapshot/"
        params = {
            'exchange': exchange,
            
            'periodicity': periodicity,
            'period': period
        }
        if nonTraded:
            params['max'] = nonTraded
        if from_timestamp:
            params['from'] = from_timestamp
        if to_timestamp:
            params['to'] = to_timestamp
        if instrumentType:
            params['instrumentType'] = instrumentType
        return self._get(endpoint, params)



from .base_api import BaseAPI

class GetProducts(BaseAPI):
    def Get_Products(self,exchange,instrumentType=None):
        endpoint = "GetProducts/"
        params = {
            'exchange': exchange
            
        }
        if instrumentType is not None:
            params['instrumentType'] = instrumentType
        
        

        return self._get(endpoint,params) 



from.base_api import BaseAPI
from.GetExchangeMessages import GetExchangeMessages
from .GetExchanges import GetExchanges
from .GetExpiryDates import GetExpiryDates
from .GetLastQuote import GetLastQuote
from .GetLastQuoteShort import GetLastQuoteShort
from .GetLastQuoteShortWithClose import GetLastQuoteShortWithClose
from .GetLastQuoteArray import GetLastQuoteArray
from .GetLastQuoteArrayShort import GetLastQuoteArrayShort
from .GetLastQuoteArrayShortWithClose import GetLastQuoteArrayShortWithClose
from .GetInstruments import GetInstruments
from .GetInstrumentsOnSearch import GetInstrumentsOnSearch
from .GetInstrumentTypes import GetInstrumentTypes
from .GetHistory import GetHistory
from .GetSnapshot import GetSnapshot
from .GetExchangeSnapshot import GetExchangeSnapshot
from .GetMarketMessages import GetMarketMessages
from .GetLimitation import GetLimitation
from .GetServerInfo import GetServerInfo
from .GetStrikePrices import GetStrikePrices
from .GetProducts import GetProducts
from .GetOptionTypes import GetOptionTypes
from .GetLastQuoteOptionChain import GetLastQuoteOptionChain
from .GetLastQuoteOptionGreeks import GetLastQuoteOptionGreeks
from .GetLastQuoteOptionGreeksChain import GetLastQuoteOptionGreeksChain
from .GetLastQuoteArrayOptionGreeks import GetLastQuoteArrayOptionGreeks

__all__ = [
    'BaseAPI', 'GetExchangeMessages', 'GetExchanges', 'GetExpiryDates', 'GetLastQuote', 
    'GetLastQuoteShort', 'GetLastQuoteShortWithClose', 'GetLastQuoteArray', 
    'GetLastQuoteArrayShort', 'GetLastQuoteArrayShortWithClose', 'GetInstruments', 
    'GetInstrumentsOnSearch', 'GetInstrumentTypes', 'GetHistory', 'GetSnapshot', 
    'GetExchangeSnapshot', 'GetMarketMessages', 'GetLimitation', 'GetServerInfo', 
    'GetStrikePrices', 'GetProducts', 'GetOptionTypes', 'GetLastQuoteOptionChain', 
    'GetLastQuoteOptionGreeks', 'GetLastQuoteOptionGreeksChain', 
    'GetLastQuoteArrayOptionGreeks'
]


from .base_api import BaseAPI

class GetOptionTypes(BaseAPI):
    def Get_OptionTypes(self,exchange,instrumentType=None,product=None,expiry=None):
        endpoint = "GetOptionTypes/"
        params = {
            'exchange': exchange
            
        }
        if instrumentType is not None:
            params['instrumentType'] = instrumentType
        if product is not None:
            params['product'] = product
        if expiry is not None:
            params['expiry'] = expiry
        

        return self._get(endpoint,params) 



from .base_api import BaseAPI

class GetHistory(BaseAPI):
    def Get_History(self, exchange, instrument_identifier, periodicity, period, max_results=None, from_timestamp=None, to_timestamp=None):
        endpoint = "GetHistory/"
        params = {
            'exchange': exchange,
            'instrumentIdentifier': instrument_identifier,
            'periodicity': periodicity,
            'period': period
        }
        if max_results:
            params['max'] = max_results
        if from_timestamp:
            params['from'] = from_timestamp
        if to_timestamp:
            params['to'] = to_timestamp
        return self._get(endpoint, params)



from .base_api import BaseAPI

class GetInstruments(BaseAPI):
    
    
    def Get_Instruments(self, exchange, product=None, instrument_type=None, option_type=None,expiry=None,StrikePrice=None, only_active=False,detailed_info=False):
        endpoint = "GetInstruments/"
        params = {
            'exchange': exchange
        }
        if product:
            params['product'] = product
        if instrument_type:
            params['instrumentType'] = instrument_type
        if option_type:
            params['optionType'] = option_type
        if expiry:
            params['expiry'] = expiry
        if StrikePrice:
            params['StrikePrice'] = StrikePrice
        if only_active:
            params['onlyActive'] = only_active
        if detailed_info:
            params['detailedInfo'] = detailed_info
        return self._get(endpoint, params)



from .base_api import BaseAPI

class GetLastQuoteOptionGreeks(BaseAPI):
    def Get_LastQuoteOptionGreeks(self,exchange,tokens,detailedInfo=None):
        endpoint = "GetLastQuoteOptionGreeks/"
        params = {
            'exchange': exchange,
            'tokens':tokens
            
        }
        if detailedInfo is not None:
            params['detailedInfo'] = detailedInfo
        

        return self._get(endpoint,params) 
 




from .base_api import BaseAPI

class GetExpiryDates(BaseAPI):
    def Get_ExpiryDates(self,exchange,product=None,instrumentType=None):
        endpoint = "GetExpiryDates/"
        params = {
            'exchange': exchange
            
        }
        if instrumentType is not None:
            params['instrumentType'] = instrumentType
        if product is not None:
            params['product'] = product
        

        return self._get(endpoint,params) 



from .base_api import BaseAPI

class GetLastQuoteArrayShortWithClose(BaseAPI):
    def Get_LastQuoteArrayShortWithClose(self, exchange, instrument_identifiers):
        endpoint = "GetLastQuoteArrayShortWithClose/"
        params = {
            'exchange': exchange,
            'instrumentIdentifiers': instrument_identifiers
        }
        return self._get(endpoint, params)


from setuptools import setup, find_packages

VERSION = '1.1.5'

DESCRIPTION = 'Beta Package to implement REST API of Global Datafeeds'
LONG_DESCRIPTION = 'Package to implement REST API of Global Datafeeds. This api will provide  data as Ondemand request ' \
                   ' as well as historical data. '
with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()
setup(
    name='restgfdl',
    version=VERSION,
    packages=find_packages(),
    install_requires=[
        'requests',
    ],
    python_requires='>=3.6',
    author='Surendran M',
    author_email='surendran.m@globaldatafeeds.in',
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type='text/markdown', 
    readme="README.md",
    
    

    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
)



from .base_api import BaseAPI

class GetSnapshot(BaseAPI):
    def Get_Snapshot(self, exchange, periodicity, period, instrument_identifiers, is_short_identifiers=None):
        endpoint = "GetSnapshot/"
        params = {
            'exchange': exchange,
            'periodicity': periodicity,
            'period': period,
            'instrumentIdentifiers': instrument_identifiers
        }
        if is_short_identifiers is not None:
            params['isShortIdentifiers'] = is_short_identifiers
        return self._get(endpoint, params)




from .base_api import BaseAPI

class GetLastQuoteShortWithClose(BaseAPI):
    def Get_LastQuoteShortWithClose(self, exchange, instrument_identifier):
        endpoint = "GetLastQuoteShortWithClose/"
        params = {
            'exchange': exchange,
            'instrumentIdentifier': instrument_identifier
        }
        return self._get(endpoint, params) 
